@@ -1,15 +1,20 @@
 from typing import Optional
 
 import docker
+from cachetools import TTLCache, cached
 
 from .config import Config
 from .data_model import *
 from .exceptions import *
 from .services import *
+from .version import VERSION
 
 __all__ = ["Beaker"]
 
 
+_UPGRADE_WARNING_ISSUED = False
+
+
 class Beaker:
     """
     A client for interacting with `Beaker <https://beaker.org>`_.
@@ -19,7 +24,11 @@ class Beaker:
     The easiest way to initialize a Beaker client is with :meth:`.from_env()`.
     """
 
-    def __init__(self, config: Config):
+    def __init__(self, config: Config, check_for_upgrades: bool = True):
+        # See if there's a newer version, and if so, suggest that the user upgrades.
+        if check_for_upgrades and not _UPGRADE_WARNING_ISSUED:
+            self._check_for_upgrades()
+
         self._config = config
         self._docker: Optional[docker.DockerClient] = None
 
@@ -28,6 +37,7 @@ class Beaker:
         self._organization = OrganizationClient(self)
         self._workspace = WorkspaceClient(self)
         self._cluster = ClusterClient(self)
+        self._node = NodeClient(self)
         self._dataset = DatasetClient(self)
         self._image = ImageClient(self)
         self._job = JobClient(self)
@@ -37,6 +47,33 @@ class Beaker:
         if self._config.default_workspace is not None:
             self.workspace.ensure(self._config.default_workspace)
 
+    @staticmethod
+    @cached(cache=TTLCache(maxsize=10, ttl=5 * 60))
+    def _check_for_upgrades():
+        import warnings
+
+        import packaging.version
+        import requests
+
+        global _UPGRADE_WARNING_ISSUED
+
+        try:
+            response = requests.get(
+                "https://api.github.com/repos/allenai/beaker-py/releases/latest", timeout=1
+            )
+            if response.ok:
+                latest_version = packaging.version.parse(response.json()["tag_name"])
+                if latest_version > packaging.version.parse(VERSION):
+                    warnings.warn(
+                        f"You're using beaker-py v{VERSION}, "
+                        f"but a newer version (v{latest_version}) is available. "
+                        f"Please upgrade with `pip install --upgrade beaker-py`.",
+                        UserWarning,
+                    )
+                    _UPGRADE_WARNING_ISSUED = True
+        except (requests.exceptions.Timeout, requests.exceptions.ConnectionError):
+            pass
+
     @classmethod
     def from_env(cls, **overrides) -> "Beaker":
         """
@@ -81,6 +118,10 @@ class Beaker:
         """
         return self._cluster
 
+    @property
+    def node(self) -> NodeClient:
+        return self._node
+
     @property
     def dataset(self) -> DatasetClient:
         """
