@@ -1,5 +1,6 @@
 import logging
 from datetime import datetime
+from enum import Enum, auto
 from typing import Any, Dict, List, Optional, Type, TypeVar
 
 from pydantic import BaseModel as _BaseModel
@@ -12,10 +13,14 @@ T = TypeVar("T")
 logger = logging.getLogger(__name__)
 
 
+SPEC_VERSION = "v2-alpha"
+
+
 class BaseModel(_BaseModel):
     class Config:
         validate_assignment = True
         alias_generator = to_lower_camel
+        use_enum_values = True
         #  extra = "forbid"
 
     @root_validator(pre=True)
@@ -53,273 +58,18 @@ class BaseModel(_BaseModel):
         return self.dict(by_alias=True, exclude_none=True)
 
 
-class WorkspaceSize(BaseModel):
-    datasets: int
-    experiments: int
-    groups: int
-    images: int
+class BaseEnum(Enum):
+    def _generate_next_value_(name, start, count, last_values):
+        return name
 
+    def __str__(self):
+        return self.value
 
-class Account(BaseModel):
-    id: str
-    name: str
-    display_name: str
-    institution: Optional[str] = None
-
-
-class Organization(BaseModel):
-    id: str
-    name: str
-    description: str
-    created: datetime
-    display_name: str
-
-
-class OrganizationMember(BaseModel):
-    role: str
-    organization: Organization
-    user: Account
-
-
-class NodeSpec(BaseModel):
-    cpu_count: Optional[float] = None
-    memory: Optional[str] = None
-    gpu_count: Optional[int] = None
-    gpu_type: Optional[str] = None
-
-
-class NodeShape(BaseModel):
-    cpu_count: Optional[float] = None
-    memory: Optional[str] = None
-    gpu_count: Optional[int] = None
-    gpu_type: Optional[str] = None
-
-
-class Cluster(BaseModel):
-    id: str
-    name: str
-    full_name: str
-    created: datetime
-    autoscale: bool
-    capacity: int
-    preemptible: bool
-    status: str
-    node_spec: NodeSpec
-    node_shape: Optional[NodeShape] = None
-    nodeCost: Optional[str] = None
-    validated: Optional[datetime] = None
-
-    @validator("validated")
-    def _validate_datetime(cls, v: Optional[datetime]) -> Optional[datetime]:
-        if v is not None and v.year == 1:
-            return None
-        return v
-
-
-class Node(BaseModel):
-    id: str
-    hostname: str
-    created: datetime
-    expiry: datetime
-    limits: NodeSpec
-
-
-class Workspace(BaseModel):
-    id: str
-    name: str
-    size: WorkspaceSize
-    owner: Account
-    author: Account
-    created: datetime
-    modified: datetime
-    archived: bool = False
-    full_name: str
-
-
-class WorkspaceRef(BaseModel):
-    id: str
-    name: str
-    full_name: str
-
-
-class ExecutionState(BaseModel):
-    created: Optional[datetime] = None
-    scheduled: Optional[datetime] = None
-    started: Optional[datetime] = None
-    exited: Optional[datetime] = None
-    finalized: Optional[datetime] = None
-    exit_code: Optional[int] = None
-
-    @validator("created", "scheduled", "started", "exited", "finalized")
-    def _validate_datetime(cls, v: Optional[datetime]) -> Optional[datetime]:
-        if v is not None and v.year == 1:
-            return None
-        return v
-
-
-class JobStatus(BaseModel):
-    created: Optional[datetime] = None
-    scheduled: Optional[datetime] = None
-    started: Optional[datetime] = None
-    exited: Optional[datetime] = None
-    finalized: Optional[datetime] = None
-    exit_code: Optional[int] = None
-
-    @validator("created", "scheduled", "started", "exited", "finalized")
-    def _validate_datetime(cls, v: Optional[datetime]) -> Optional[datetime]:
-        if v is not None and v.year == 1:
-            return None
-        return v
-
-
-class ExecutionResult(BaseModel):
-    beaker: str
-
-
-class ExecutionLimits(BaseModel):
-    cpu_count: Optional[float] = None
-    memory: Optional[str] = None
-    gpus: List[str] = Field(default_factory=list)
-
-
-class Execution(BaseModel):
-    id: str
-    task: str
-    experiment: str
-    workspace: str
-    author: Account
-    spec: Dict[str, Any]
-    result: ExecutionResult
-    state: ExecutionState
-    node: Optional[str] = None
-    limits: Optional[ExecutionLimits] = None
-    priority: str = "normal"
-
-
-class JobRequests(BaseModel):
-    gpu_count: Optional[int] = None
-    cpu_count: Optional[float] = None
-    memory: Optional[str] = None
-    sharedMemory: Optional[str] = None
-
-
-class JobLimits(BaseModel):
-    cpu_count: Optional[float] = None
-    memory: Optional[str] = None
-    gpus: List[str] = Field(default_factory=list)
-
-
-class JobExecution(BaseModel):
-    task: str
-    experiment: str
-    workspace: str
-    spec: Dict[str, Any]
-    result: ExecutionResult
-
-
-class Job(BaseModel):
-    id: str
-    kind: str
-    name: str
-    author: Account
-    workspace: str
-    cluster: str
-    status: JobStatus
-    execution: JobExecution
-    node: Optional[str] = None
-    requests: Optional[JobRequests] = None
-    limits: Optional[JobLimits] = None
-
-
-class Experiment(BaseModel):
-    id: str
-    name: str
-    full_name: str
-    owner: Account
-    author: Account
-    created: datetime
-    workspace_ref: WorkspaceRef
-    executions: List[Execution] = Field(default_factory=list)
-    jobs: List[Job] = Field(default_factory=list)
-
-
-class DatasetStorage(BaseModel):
-    id: str
-    address: str
-    token: str
-    token_expires: datetime
-
-
-class DatasetSize(BaseModel):
-    final: bool
-    files: int
-    bytes: int
-    bytes_human: str
-
-
-class Dataset(BaseModel):
-    id: str
-    owner: Account
-    author: Account
-    created: datetime
-    workspace_ref: WorkspaceRef
-    committed: Optional[datetime] = None
-    name: Optional[str] = None
-    full_name: Optional[str] = None
-    storage: Optional[DatasetStorage] = None
-
-    @validator("committed")
-    def _validate_datetime(cls, v: Optional[datetime]) -> Optional[datetime]:
-        if v is not None and v.year == 1:
-            return None
-        return v
-
-
-class DatasetStorageInfo(BaseModel):
-    id: str
-    created: Optional[datetime] = None
-    size: Optional[DatasetSize] = None
-    readonly: bool = True
-
-    @validator("created")
-    def _validate_datetime(cls, v: Optional[datetime]) -> Optional[datetime]:
-        if v is not None and v.year == 1:
-            return None
-        return v
-
-
-class FileInfo(BaseModel):
-    path: str
-    size: int
-    digest: str
-    updated: datetime
-    url: str
-
-
-class DatasetManifest(BaseModel):
-    files: List[FileInfo]
-    cursor: Optional[str] = None
-
-
-class Image(BaseModel):
-    id: str
-    name: str
-    full_name: str
-    original_tag: str
-    owner: Account
-    author: Account
-    created: datetime
-    workspace_ref: WorkspaceRef
-    committed: Optional[datetime] = None
-
-    @validator("committed")
-    def _validate_datetime(cls, v: Optional[datetime]) -> Optional[datetime]:
-        if v is not None and v.year == 1:
-            return None
-        return v
-
-
-SPEC_VERSION = "v2-alpha"
+    def __eq__(self, o: object):
+        if isinstance(o, str):
+            return self.value == o
+        else:
+            return super().__eq__(o)
 
 
 class ImageSource(BaseModel):
@@ -430,7 +180,6 @@ class DataSource(BaseModel):
 
     @root_validator(pre=True)
     def _check_exactly_one_field_set(cls, values: Dict[str, Any]) -> Dict[str, Any]:
-        print(values)
         if len([v for v in values.values() if v is not None]) != 1:
             raise ValueError("Exactly one data source field must be set.")
         return values
@@ -560,12 +309,6 @@ class TaskSpec(BaseModel):
     creating an execution graph.
     """
 
-    name: str
-    """
-    Name is used for display and to refer to the task throughout the spec.
-    It must be unique among all tasks within its experiment.
-    """
-
     image: ImageSource
     """
     A base image to run, usually built with Docker.
@@ -581,6 +324,12 @@ class TaskSpec(BaseModel):
     Context describes how and where this task should run.
     """
 
+    name: Optional[str] = None
+    """
+    Name is used for display and to refer to the task throughout the spec.
+    It must be unique among all tasks within its experiment.
+    """
+
     command: Optional[List[str]] = None
     """
     Command is the full shell command to run as a list of separate arguments.
@@ -642,3 +391,293 @@ class ExperimentSpec(BaseModel):
         if v != SPEC_VERSION:
             raise ValueError(f"Only version '{SPEC_VERSION}' is currently supported")
         return v
+
+
+class WorkspaceSize(BaseModel):
+    datasets: int
+    experiments: int
+    groups: int
+    images: int
+
+
+class Account(BaseModel):
+    id: str
+    name: str
+    display_name: str
+    institution: Optional[str] = None
+
+
+class Organization(BaseModel):
+    id: str
+    name: str
+    description: str
+    created: datetime
+    display_name: str
+
+
+class OrganizationMember(BaseModel):
+    role: str
+    organization: Organization
+    user: Account
+
+
+class NodeSpec(BaseModel):
+    cpu_count: Optional[float] = None
+    memory: Optional[str] = None
+    gpu_count: Optional[int] = None
+    gpu_type: Optional[str] = None
+
+
+class NodeShape(BaseModel):
+    cpu_count: Optional[float] = None
+    memory: Optional[str] = None
+    gpu_count: Optional[int] = None
+    gpu_type: Optional[str] = None
+
+
+class NodeSpecUtil(BaseModel):
+    cpu_count: Optional[float] = None
+    gpu_count: Optional[int] = None
+    #  memory: Optional[str]   # TODO
+
+
+class Cluster(BaseModel):
+    id: str
+    name: str
+    full_name: str
+    created: datetime
+    autoscale: bool
+    capacity: int
+    preemptible: bool
+    status: str
+    node_spec: NodeSpec
+    node_shape: Optional[NodeShape] = None
+    nodeCost: Optional[str] = None
+    validated: Optional[datetime] = None
+
+    @validator("validated")
+    def _validate_datetime(cls, v: Optional[datetime]) -> Optional[datetime]:
+        if v is not None and v.year == 1:
+            return None
+        return v
+
+
+class Node(BaseModel):
+    id: str
+    hostname: str
+    created: datetime
+    expiry: datetime
+    limits: NodeSpec
+
+
+class NodeUtilization(BaseModel):
+    id: str
+    hostname: str
+    limits: NodeSpec
+    used: NodeSpecUtil
+    free: NodeSpecUtil
+
+
+class Workspace(BaseModel):
+    id: str
+    name: str
+    size: WorkspaceSize
+    owner: Account
+    author: Account
+    created: datetime
+    modified: datetime
+    archived: bool = False
+    full_name: str
+
+
+class WorkspaceRef(BaseModel):
+    id: str
+    name: str
+    full_name: str
+
+
+class CurrentJobStatus(BaseEnum):
+    created = auto()
+    scheduled = auto()
+    running = auto()
+    idle = auto()
+    exited = auto()
+    finalized = auto()
+
+
+class JobStatus(BaseModel):
+    created: Optional[datetime] = None
+    scheduled: Optional[datetime] = None
+    started: Optional[datetime] = None
+    exited: Optional[datetime] = None
+    failed: Optional[datetime] = None
+    finalized: Optional[datetime] = None
+    canceled: Optional[datetime] = None
+    idle_since: Optional[datetime] = None
+    exit_code: Optional[int] = None
+    message: Optional[str] = None
+
+    @validator(
+        "created", "scheduled", "started", "exited", "failed", "finalized", "canceled", "idle_since"
+    )
+    def _validate_datetime(cls, v: Optional[datetime]) -> Optional[datetime]:
+        if v is not None and v.year == 1:
+            return None
+        return v
+
+    @property
+    def current(self) -> CurrentJobStatus:
+        """
+        Get the :class:`CurrentJobStatus`.
+
+        :raises ValueError: If status can't be determined.
+        """
+        if self.finalized is not None:
+            return CurrentJobStatus.finalized
+        elif self.exited is not None:
+            return CurrentJobStatus.exited
+        elif self.idle_since is not None:
+            return CurrentJobStatus.idle
+        elif self.started is not None:
+            return CurrentJobStatus.running
+        elif self.scheduled is not None:
+            return CurrentJobStatus.scheduled
+        elif self.created is not None:
+            return CurrentJobStatus.created
+        else:
+            raise ValueError(f"Invalid status {self}")
+
+
+class ExecutionResult(BaseModel):
+    beaker: str
+
+
+class JobRequests(BaseModel):
+    gpu_count: Optional[int] = None
+    cpu_count: Optional[float] = None
+    memory: Optional[str] = None
+    sharedMemory: Optional[str] = None
+
+
+class JobLimits(BaseModel):
+    cpu_count: Optional[float] = None
+    memory: Optional[str] = None
+    gpus: List[str] = Field(default_factory=list)
+
+
+class JobExecution(BaseModel):
+    task: str
+    experiment: str
+    spec: TaskSpec
+    result: ExecutionResult
+
+
+class JobKind(BaseEnum):
+    execution = auto()
+    session = auto()
+
+
+class Job(BaseModel):
+    id: str
+    kind: JobKind
+    author: Account
+    workspace: str
+    cluster: str
+    status: JobStatus
+    execution: Optional[JobExecution] = None
+    name: Optional[str] = None
+    node: Optional[str] = None
+    requests: Optional[JobRequests] = None
+    limits: Optional[JobLimits] = None
+
+
+class Jobs(BaseModel):
+    data: Optional[List[Job]] = None
+    next: Optional[str] = None
+
+
+class Experiment(BaseModel):
+    id: str
+    name: str
+    full_name: str
+    owner: Account
+    author: Account
+    created: datetime
+    workspace_ref: WorkspaceRef
+    jobs: List[Job] = Field(default_factory=list)
+
+
+class DatasetStorage(BaseModel):
+    id: str
+    address: str
+    token: str
+    token_expires: datetime
+
+
+class DatasetSize(BaseModel):
+    final: bool
+    files: int
+    bytes: int
+    bytes_human: str
+
+
+class Dataset(BaseModel):
+    id: str
+    owner: Account
+    author: Account
+    created: datetime
+    workspace_ref: WorkspaceRef
+    committed: Optional[datetime] = None
+    name: Optional[str] = None
+    full_name: Optional[str] = None
+    storage: Optional[DatasetStorage] = None
+
+    @validator("committed")
+    def _validate_datetime(cls, v: Optional[datetime]) -> Optional[datetime]:
+        if v is not None and v.year == 1:
+            return None
+        return v
+
+
+class DatasetStorageInfo(BaseModel):
+    id: str
+    created: Optional[datetime] = None
+    size: Optional[DatasetSize] = None
+    readonly: bool = True
+
+    @validator("created")
+    def _validate_datetime(cls, v: Optional[datetime]) -> Optional[datetime]:
+        if v is not None and v.year == 1:
+            return None
+        return v
+
+
+class FileInfo(BaseModel):
+    path: str
+    size: int
+    digest: str
+    updated: datetime
+    url: str
+
+
+class DatasetManifest(BaseModel):
+    files: List[FileInfo]
+    cursor: Optional[str] = None
+
+
+class Image(BaseModel):
+    id: str
+    name: str
+    full_name: str
+    original_tag: str
+    owner: Account
+    author: Account
+    created: datetime
+    workspace_ref: WorkspaceRef
+    committed: Optional[datetime] = None
+
+    @validator("committed")
+    def _validate_datetime(cls, v: Optional[datetime]) -> Optional[datetime]:
+        if v is not None and v.year == 1:
+            return None
+        return v
